/*
 * Copyright (c) 2011 Marc R. Hoffmann, hoffmann@mountainminds.com
 * 
 * This work is licensed under the Creative Commons CC BY-SA License. To view a
 * copy of this license, visit http://creativecommons.org/licenses/by-sa/3.0/
 */
package com.mountainminds.examples.java7.jvm;

import java.lang.invoke.CallSite;
import java.lang.invoke.ConstantCallSite;
import java.lang.invoke.MethodHandle;
import java.lang.invoke.MethodHandles.Lookup;
import java.lang.invoke.MethodType;

import org.junit.Before;
import org.junit.Test;

/**
 * 
 * 
 * 
 */
public class InvokeDynamicTest {

	private InvokeDynamicFactory factory;

	@Before
	public void setup() {
		factory = new InvokeDynamicFactory(getClass().getClassLoader());
	}

	/** This is the base class we use to define the desired interface */
	public static abstract class AbstractPrinter {
		public abstract void print(String text);
	}

	@Test
	public void testPrint() throws Exception {
		AbstractPrinter printer = factory
				.create(AbstractPrinter.class,
						"com/mountainminds/examples/java7/jvm/InvokeDynamicTest",
						"bootstrap",
						"(Ljava/lang/invoke/MethodHandles$Lookup;Ljava/lang/String;Ljava/lang/invoke/MethodType;[Ljava/lang/Object;)Ljava/lang/invoke/CallSite;");

		printer.print("Hello ");
		printer.print("World!");
	}

	// The actual implementation of the method we're going to call
	public static void printImpl(String text) {
		System.out.println(text);
	}

	// The bootstrap method is called once to resolve the call site
	public static CallSite bootstrap(Lookup caller, String name,
			MethodType type, Object... args) throws Exception {

		System.out.println("bootstrap for: " + name + type);

		MethodHandle handle = caller.findStatic(InvokeDynamicTest.class,
				"printImpl", MethodType.methodType(Void.TYPE, String.class));

		return new ConstantCallSite(handle);
	}

}
