/*
 * Copyright (c) 2011 Marc R. Hoffmann, hoffmann@mountainminds.com
 * 
 * This work is licensed under the Creative Commons CC BY-SA License. To view a
 * copy of this license, visit http://creativecommons.org/licenses/by-sa/3.0/
 */
package com.mountainminds.examples.java7.jvm;

import java.util.HashMap;
import java.util.Map;

/**
 * Internal class loader to load classes directly from memory.
 */
class MemoryClassLoader extends ClassLoader {

	private final Map<String, byte[]> definitions = new HashMap<String, byte[]>();

	public MemoryClassLoader(ClassLoader parent) {
		super(parent);
	}

	public void add(String name, byte[] def) {
		definitions.put(name, def);
	}

	@Override
	protected Class<?> findClass(String name) throws ClassNotFoundException {
		byte[] def = definitions.get(name);
		if (def == null) {
			throw new ClassNotFoundException(name);
		} else {
			return defineClass(name, def, 0, def.length);
		}
	}

	/**
	 * Convenience method to load a single class from memory.
	 * 
	 * @throws ClassNotFoundException
	 */
	public static Class<?> load(ClassLoader parent, String name, byte[] def)
			throws ClassNotFoundException {
		MemoryClassLoader cl = new MemoryClassLoader(parent);
		cl.add(name, def);
		return cl.loadClass(name);
	}

}